package com.example.haircosmeticsanalyser

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Bundle
import android.text.method.ScrollingMovementMethod
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat.requestPermissions
import androidx.core.content.ContextCompat.checkSelfPermission
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModelProviders


/**
 * A simple [Fragment] subclass as the second destination in the navigation.
 */
class CameraFragment : Fragment() {

    lateinit var viewModel: MainActivityViewModel
    lateinit var searchIngredient: String

    var fileName = "files/cosmetic1.txt"
    var isDescriptionVisible = false

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_camera, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        (activity as AppCompatActivity).supportActionBar?.show()
        (activity as AppCompatActivity?)!!.supportActionBar!!.title = getString(R.string.camera)

        val context = activity?.getApplicationContext()!!
        viewModel = ViewModelProviders.of(this).get(MainActivityViewModel::class.java)

        view.findViewById<Button>(R.id.btnPickImage).setOnClickListener {
            if (checkSelfPermission(
                    requireContext(),
                    android.Manifest.permission.READ_EXTERNAL_STORAGE
                )
                != PackageManager.PERMISSION_GRANTED) {
                    requestPermissions(
                        requireActivity(),
                        arrayOf(android.Manifest.permission.READ_EXTERNAL_STORAGE), PERMISSION_CODE
                    )
            } else {
                pickImageFromGallery()
            }
        }

        view.findViewById<Button>(R.id.btnAnalyseImage).setOnClickListener {
            var inci = view.context.assets.open(fileName).bufferedReader().use {
                it.readText()
            }
            var list =  inci.split(", ", ".")
            list.toMutableList()

            var finalMap : Map<String, String> = list.map { it to " " }.toMap()
            var mutableFinalMap = finalMap.toMutableMap()


            val txtViewAnalyseImageResponse = view.findViewById<TextView>(R.id.txtViewAnalyseImageResponse)
            var inciAnalysis : String = ""

            mutableFinalMap.forEach{ (key, value) ->
                searchIngredient = key
                var searchIngredientResponse = viewModel.getSearchResult(searchIngredient)
                if (searchIngredientResponse != null) {
                    mutableFinalMap[key] = searchIngredientResponse.category
                    inciAnalysis +=  searchIngredientResponse.name + " - " + searchIngredientResponse.category + " \n" + searchIngredientResponse.description + "\n\n"
                }
            }

            txtViewAnalyseImageResponse.text = inciAnalysis

            view.findViewById<TextView>(R.id.txtViewAnalyseImageTitle).visibility = View.VISIBLE
            view.findViewById<TextView>(R.id.txtViewAnalyseImageResponse).visibility = View.GONE
            view.findViewById<TextView>(R.id.txtViewAnalyseImageSummaryTitle).visibility = View.VISIBLE
            view.findViewById<TextView>(R.id.txtViewAnalyseImageSummary).visibility = View.VISIBLE
            view.findViewById<TextView>(R.id.txtViewAnalyseImageSummary).text = summaryAlgorithm(context, mutableFinalMap)
        }

        view.findViewById<TextView>(R.id.txtViewAnalyseImageTitle).setOnClickListener {
            val description = view.findViewById<TextView>(R.id.txtViewAnalyseImageResponse)

            if (isDescriptionVisible) {
                description.visibility = View.GONE
                isDescriptionVisible = false
            } else {
                description.visibility = View.VISIBLE
                isDescriptionVisible = true
            }
        }

        view.findViewById<TextView>(R.id.txtViewAnalyseImageResponse).movementMethod =
            ScrollingMovementMethod()
    }

    private fun pickImageFromGallery() {
        val intent = Intent(Intent.ACTION_PICK)
        intent.type = "image/*"
        startActivityForResult(intent, IMAGE_PICK_CODE)
    }

    private fun summaryAlgorithm(context: Context, map: MutableMap<String, String>) : String {
        var list = map.values.toList()
        var fragPosition = list.indexOf("zapach")
        var poorCompositionText = ""
        var firstActiveIngredient = ""

        for (i in 0..(fragPosition/2)) {
            if (list[i] != "inne") {
                break
            }
            if (i == fragPosition/2-1) {
                poorCompositionText = context.getString(R.string.poor_composition)
            }
        }

        for (i in 0..fragPosition) {
            if (list[i] != "inne") {
                firstActiveIngredient = list[i]
                break
            }
        }

        var emolients = list.count{it == "emolient"}
        var humectants = list.count{it == "humektant"}
        var proteins = list.count{it == "proteina"}

        return if (proteins > 0) {
            poorCompositionText + context.getString(R.string.protein)
        } else if (humectants > emolients) {
            poorCompositionText + context.getString(R.string.humectant)
        } else if (humectants == 0 && proteins == 0) {
            poorCompositionText + context.getString(R.string.emollient)
        } else if (firstActiveIngredient == "emolient") {
            poorCompositionText + context.getString(R.string.emollient_humectant)
        } else if (firstActiveIngredient == "humektant") {
            poorCompositionText + context.getString(R.string.humectant_emollient)
        } else {
            ""
        }
    }

    companion object {
        private val IMAGE_PICK_CODE = 1000
        private val PERMISSION_CODE = 1001
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>,
        grantResults: IntArray
    ) {
            when(requestCode) {
                PERMISSION_CODE -> {
                    if (grantResults.size > 0 && grantResults[0] ==
                        PackageManager.PERMISSION_GRANTED
                    ) {
                        pickImageFromGallery()
                    } else {
                        Toast.makeText(context, "Permission denied", Toast.LENGTH_SHORT).show()
                    }
                }
            }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (resultCode == Activity.RESULT_OK && requestCode == IMAGE_PICK_CODE) {
            view?.findViewById<ImageView>(R.id.imgViewImage)?.setImageURI(data?.data)
        }
    }
}